#!/usr/bin/env ruby

raise "create-fai-vm must be run as root." unless Process.uid == 0

# Quick wrapper around fai-diskimage to have some sane defaults and logic.

require "optimist"

require "fileutils"

require "fai-toolkit/common"

class CreateFaiVm
  include Common

  def initialize
    # TODO: proper FAI server needs to be set even for testing, as that’s how systems would communicate in real life.
    # Actually, maybe above doesn’t matter much. Maybe? Maybe it’s just for fai-monitor.
    @server = "localhost"

    # TODO: for some reason this isn’t read from /etc/fai/nfsroot.conf, and it’s not present anywhere else.
    # I think this warrants more investigation if time allows.
    #set FAI_DEBOOTSTRAP "cskoeln /srv/fai/mirrorroot/srv/fai/mirror/debian_buster"
    #set FAI_DEBOOTSTRAP "buster http://deb.debian.org/debian"
    @fai_debootstrap = "buster http://ryza.smar.fi:3142/debian/debian"

    # Instead of this, there could be a manipulated DNS server, which returns correct IP, but since this is my local setup,
    # it doesn’t make much sense.
    @nfsroot_etc_hosts = "192.168.1.230   ryza.smar.fi"
  end

  def options
    options = Optimist::options do
      opt :image_name,        "Path to resulting image. For example, image.qcow2",
          type: :string,      short: "n"
      opt :class,             "Includes given class in image creation",
          type: :string,      short: "c",                   multi: true
      opt :size,              "Size of resulting image. For example, 20G.",
          type: :string,      short: "s", default: "10G"
      opt :hostname,          "Hostname that is used to generate the image.",
          type: :string,      short: "h"
      opt :force,             "Overwrite existing image.",
                              short: "f"
      opt :verbose,           "Show verbose output",
                              short: "v"
      opt :debug,             "Show debug output",
                              short: "d"
    end

    set_opts options

    raise "--image-name must be given. " if options[:image_name].nil?
    @image_name = options[:image_name]

    if File.exist? @image_name
      unless options[:force]
        raise "File exists at target #{@image_name} already"
      else
        FileUtils.rm @image_name
      end
    end

    @classes = options[:class]
    raise "At least one --class must be given." if @classes.nil? or @classes.empty?

    @image_size = options[:size]

    raise "--hostname must be given." if options[:hostname].nil?
    @hostname = options[:hostname]
  end

  def perform
    execute %{env -i PATH="/usr/bin:/usr/sbin" SERVER="#{@server}" LOGUSER= FAI_LOGPROTO=none FAI_DEBOOTSTRAP="#{@fai_debootstrap}" NFSROOT_ETC_HOSTS="#{@nfsroot_etc_hosts}" \
            fai-diskimage -v --hostname "#{@hostname}" -S"#{@image_size}" -c"#{@classes.join(",")}" "#{@image_name}"}, direct_output: true
  end

end

vm = CreateFaiVm.new
vm.options
vm.perform