Feature: Sometimes people need to create a new install medium to install something to a host.

  Rule: Debian 10 media creation

    Background:
      Given I have distribution "Debian 10"
      And I have existing nfsroot with following attributes:
        | dracut        | true      |
      And I have a FAI mirror

    @mounts
    Scenario: Creating a new Debian 10 installation medium should work without problems
      When I create install medium with following data:
        | image_name    | test.iso  |
        | distribution  | debian    |
        | dist_version  | 10        |
      Then I should have medium "test.iso"
      And medium "test.iso" should contain file "FAI-INSTALL-CD"
      When I mount squashfs filesystem inside medium "test.iso"
      Then mounted squashfs filesystem should contain directory "media/mirror/dists/cskoeln"

    @mounts
    Scenario: Creating a new update medium should work without problems
      When I create update medium with following data:
        | image_name    | test.iso  |
        | distribution  | debian    |
        | dist_version  | 10        |
      Then I should have medium "test.iso"
      And medium "test.iso" should contain file "FAI-UPDATE-CD"
      When I mount squashfs filesystem inside medium "test.iso"
      Then mounted squashfs filesystem should contain directory "media/mirror/dists/cskoeln"

    @mounts
    Scenario: Creating a medium with host config should work
      Given I have host configuration with following data:
        | target_site   | test_site   |
        | target_host   | test_host   |
        | image_name    | test.iso    |
        | distribution  | debian      |
        | dist_version  | 10          |
      When I create a medium for site "test_site" and host "test_host"
      Then I should have medium "test.iso"
      And medium "test.iso" should contain file "FAI-CD"
      When I mount squashfs filesystem inside medium "test.iso"
      Then mounted squashfs filesystem should contain directory "media/mirror/dists/cskoeln"

  Rule: OpenSUSE 15.2 media creation

    # TODO: There should be a test that tests OpenSUSE install CD creation using Debian nfsroot.
    # That should work, because the installation can work then by base archive.

    Background:
      Given I have distribution "OpenSUSE 15.2"
      And I have existing nfsroot with following attributes:
        | dracut        | true      |
      And I have a FAI mirror

    @mounts
    Scenario: Creating a new OpenSUSE installation medium should work without problems
      When I create install medium with following data:
        | image_name    | test.iso  |
        | distribution  | opensuse  |
        | dist_version  | 15.2      |
      Then I should have medium "test.iso"
      And medium "test.iso" should contain file "FAI-INSTALL-CD"
      When I mount squashfs filesystem inside medium "test.iso"
      Then mounted squashfs filesystem should contain directory "media/mirror/opensuse/var/cache/zypp"


  Rule: Mirror should be usable inside an nfsroot
    The point of the FAI mirror is to have minimal set of packages that can be used inside nfsroot,
    for offline installations

    @mounts
    Scenario Outline: A FAI install CD should be able to use a FAI mirror bundled in the CD.
      Given I have distribution "<distribution_name>"
      And I have existing nfsroot with following attributes:
        | dracut        | <dracut>            |
      And I have a FAI mirror
      And I have install medium with following data:
        | image_name    | <image_name>        |
        | distribution  | <distribution_slug> |
        | dist_version  | <dist_version>      |
      When I mount squashfs filesystem inside medium "<image_name>"
      And I chroot to mounted filesystem
      And I use only fai_cd.repo
      And I refresh package lists
      And I install package "traceroute"
      Then I should have package "traceroute" installed
      Then I should close chroot session

      # TODO: Cucumber syntax file should handle this case also.
      Examples:
        | distribution_name   | dracut  | image_name              | distribution_slug | dist_version  |
        | OpenSUSE 15.2       | true    | test-opensuse-15.2.iso  | opensuse          | 15.2          |
