# Steps for FAI mirror creation.

require_relative "../../lib/common"

include Common

require_relative "../../lib/create-fai-mirror/fai-mirror"

def create_fai_mirror fai_config_dir, target_distribution, target_distribution_version, opts = {}

  # For testing, have our newest files in the nfsroot, so we don’t test against old files.
  if opts[:nfsroot_path]
    config = @command_object.instance_variable_get :@config
    expect(config).to be

    case @distribution_id
    when "debian"
      fai_toolkit_path = "/usr/lib/ruby/vendor_ruby/fai-toolkit"
      unless Dir.exist? "#{config.nfsroot}#{fai_toolkit_path}"
        raise "fai-toolkit is not installed in nfsroot"
      end

      FileUtils.cp_r Dir.glob("#{__dir__}/../../lib/*"), "#{config.nfsroot}#{fai_toolkit_path}"
      FileUtils.cp "#{__dir__}/../../bin/create-fai-mirror.rb", "#{config.nfsroot}/usr/sbin/create-fai-mirror"
      FileUtils.chmod 755, "#{config.nfsroot}/usr/sbin/create-fai-mirror"
    when "opensuse-leap"
      fai_toolkit_path = "/usr/lib64/ruby/site_ruby/fai-toolkit"
      unless Dir.exist? "#{config.nfsroot}#{fai_toolkit_path}"
        raise "fai-toolkit is not installed in nfsroot"
      end

      FileUtils.cp_r Dir.glob("#{__dir__}/../../lib/*"), "#{config.nfsroot}#{fai_toolkit_path}"
      FileUtils.cp "#{__dir__}/../../bin/create-fai-mirror.rb", "#{config.nfsroot}/usr/sbin/create-fai-mirror"
      FileUtils.chmod 755, "#{config.nfsroot}/usr/sbin/create-fai-mirror"
    else
      raise "Not implemented for distro #{@distribution_name}"
    end
  end

  ARGV.clear
  #ARGV << "--verbose"
  #ARGV << "--verbose" << "--debug"
  ARGV << "--fai-config-dir" << fai_config_dir
  ARGV << "--target-distribution" << target_distribution
  (ARGV << "--target-distribution-version" << target_distribution_version) if target_distribution_version
  (ARGV << "--run-inside-nfsroot" << opts[:nfsroot_path]) if opts[:nfsroot_path]
  ARGV << "--mirror-location" << $test_fai_mirror

  # Exclude some distro specific classes to cause Debian things work better.
  case @distribution_id
  when "debian"
    ARGV << "--exclude-class" << "UBUNTU"
  end

  ARGV.flatten!

  log "Command: “create-fai-mirror #{ARGV.join(" ")}”"

  updatecd = CreateFaiMirror::FaiMirror.new

  begin
    updatecd.options
  rescue SystemExit => e
    # Ran inside nfsroot, so can skip to next step.
    if e.success?
      @fai_mirror_path = "#{config.nfsroot}/#{$test_fai_mirror}/#{@distribution_slug}"
      Dir.chdir "#{__dir__}/../.."
      return
    end
    raise e
  end

  # NOTE: I’m not actually sure if this is used for anything real, but too lazy to test now.
  if opts[:host_configurations]
    config = updatecd.instance_variable_get :@config
    expect(config).to be

    config.instance_variable_set :host_configurations, opts[:host_configurations]

    puts config.host_configurations.inspect
    exit 1
  end

  updatecd.perform

  @fai_mirror_path = "#{$test_fai_mirror}/#{@distribution_slug}"

  # Reset current working dir to make directory resolution work properly.
  Dir.chdir "#{__dir__}/../.."
end

# Similar to “I have existing nfsroot with following attributes:” step,
# this first checks if mirror is present and only then creates one.
Given('I have a FAI mirror') do
  config = @command_object.instance_variable_get :@config
  expect(config).to be

  if Dir.exist? "#{config.nfsroot}/#{$test_fai_mirror}"
    log "Using existing FAI mirror."
    @fai_mirror_path = "#{config.nfsroot}/#{$test_fai_mirror}/#{@distribution_slug}"
    next
  end

  case @distribution_name
  when "Debian 10"
    fai_config_dir = "#{__dir__}/../../test/fai_config_dir"
    target_distribution = "debian"
  when "OpenSUSE 15.2"
    fai_config_dir = "#{__dir__}/../../test/fai_config_dirs/opensuse"
    target_distribution = "opensuse"
    target_distribution_version = "15.2"
  else
    raise "Unknown distribution #{@distribution_name}"
  end

  opts = { nfsroot_path: fai_config_dir,
           host_configurations: "#{__dir__}/../../test/host_configurations"
  }
  create_fai_mirror fai_config_dir, target_distribution, target_distribution_version, opts
end

When('I create a FAI mirror') do
  # Mirror can only be created if necessary utilities are available. In such case, better just skip whole test.
  case @distribution_name
  when "Debian 10"
    unless command_available? "apt-get"
      log "This test can only be run when apt-get is available in the system."
      pending
    end
    fai_config_dir = "#{__dir__}/../../test/fai_config_dir"
    target_distribution = "debian"
  when "OpenSUSE 15.2"
    unless command_available? "zypper"
      log "This test can only be run when zypper is available in the system."
      pending
    end
    fai_config_dir = "#{__dir__}/../../test/fai_config_dirs/opensuse"
    target_distribution = "opensuse"
    target_distribution_version = "15.2"
  else
    raise "Unknown distribution #{@distribution_name}"
  end

  create_fai_mirror fai_config_dir, target_distribution, target_distribution_version
end

When('I create a FAI mirror using nfsroot') do
  log "TODO: Need to implement some way to construct nfsroot for mirror creation"
  pending

  case @distribution_name
  when "Debian 10"
    fai_config_dir = "#{__dir__}/../../test/fai_config_dir"
    target_distribution = "debian"
  when "OpenSUSE 15.2"
    fai_config_dir = "#{__dir__}/../../test/fai_config_dirs/opensuse"
    target_distribution = "opensuse"
    target_distribution_version = "15.2"
  else
    raise "Unknown distribution #{@distribution_name}"
  end

  create_fai_mirror fai_config_dir, target_distribution, target_distribution_version, { nfsroot_path: fai_config_dir }
end

Then('in the mirror I should have package {string}') do |package|
  pending # Write code here that turns the phrase above into concrete actions
end

