
require "fileutils"
require "tmpdir"

require_relative "base"
require_relative "../options"
require_relative "../fai-config"
require_relative "nfsroot-base-methods"

# WARNING: This script won’t run if the nfsroot is already generated.
# Also you probably don’t want to use this script, unless you know what you’re doing.
class FaiToolkit::Base::NfsrootBase
  include Options
  include FaiToolkit::Base::NfsrootBaseMethods

  def options parser
    common_options parser: parser, log_file: "create_base.log" do |p|
      p.opt :filename,          "Path to the file to be written.",
          type: :string,                              short: "F"
      p.opt :fai_config_dir,    "FAI’s main config files.",
          type: :string,      default: "/etc/fai",    short: "c"
      p.opt :force,             "Overwrite existing base image.",
                              default: false,         short: "f"
    end
  end

  def check_options
    options = @options

    parse_common_options options

    raise "--filename is required." if options[:filename].nil?
    @base_path = options[:filename]
    raise "File #{@base_path} already exists. You can use --force to overwrite existing file." if File.exist? @base_path and options[:force] != true

    set_opts options
  end

  def perform
    startup_checks

    tmpdir = Dir.mktmpdir "create-base_"
    at_exit do
      FileUtils.remove_entry_secure tmpdir
    end
    # Hack FaiConfig to have tempdir as nfsroot.
    @config.instance_variable_set :@nfsroot, tmpdir
    self.nfsroot= tmpdir

    # I find it strange that the cleanup is necessary, but eh... I have a mistake somewhere.
    #umount_dirs

    resolve_sources
    create_base

    # Wait for compression to be done.
    Process.wait

    path = "#{nfsroot}/var/tmp/base.tar.xz"

    # Move the file to the location one wants it to be.
    FileUtils.mv path, @base_path
  end

  private

  def startup_checks
    @config = FaiConfig.new @fai_config_dir
    @config.load_nfsroot_conf
  end
end

