
require_relative "execution"
require_relative "fai-toolkit"
require_relative "output"

# Common utilities

module Common
  include FaiToolkit::Execution
  include Output

  # Everything should only be usable for the class inheriting this module.
  private

  @@opts = nil
  @@nfsroot = nil
  @@rootcmd = nil

  def opts
    @@opts
  end

  def set_opts options
    @@opts = options

    @@verbose = ""
    if options[:verbose]
      @@verbose = "-v"
      @@_verbose = true
    end

    @@_debug = true if options[:debug]
  end

  def rootcmd
    raise "rootcmd is not set." if @@rootcmd.nil?
    @@rootcmd
  end

  def rootcmd= cmd
    @@rootcmd = cmd
  end

  def nfsroot
    raise "nfsroot is not set." if @@nfsroot.nil?
    @@nfsroot
  end

  def nfsroot= path
    set_nfsroot path
  end

  def set_nfsroot path
    @@nfsroot = path
    @@rootcmd = %{chroot "#{@@nfsroot}" env LC_ALL="C"}
  end

  # Checks if given directory is a git repository, and if it is, raises an error if there is uncommitted changes.
  def ensure_clean_state directory
    @git_exists ||= command_available? "git"
    return unless @git_exists
    return unless File.directory? directory

    Dir.chdir(directory) do
      return if execvalue("git rev-parse --is-inside-work-tree") != "true"

      result = execvalue "git diff-index --exit-code HEAD ."
      if $?.exitstatus == 1
        raise "There is uncommitted changes in #{directory}. Please commit your changes and try again."
      end
    end
  end
end
