
require "fileutils"

require_relative "create-fai-mirror"

module CreateFaiMirror::Methods

  # removes/excludes all classes in $* from @classes
  def exclude_classes excluded_classes
    @classes.delete_if do |c|
      excluded_classes.include? c
    end
    ENV["classes"] = @classes.join("\n")
  end

  def umount_dirs
    return if @config.fai_debmirror.nil? or @config.fai_debmirror.empty?
    execute %{umount "#{@mirror_location}"}, fail: false, log: false
  end

  def cleandirs
    return # currently nothing to do
  end

  # now delete all packages that are already included in base.tar.xz
  def delete_base_packages
    base_packages_file = "#{nfsroot}/var/tmp/base-pkgs.lis"
    unless File.exist? base_packages_file
        debug "#{base_packages_file} not available."
        verb "Can't remove wasteful packages that are already in base.tar.xz."
        return
    end

    verb "Removing packages that are already included in base.tar.xz"

    File.readlines(base_packages_file).each do |p|
        all = p.gsub(":", " ").split(" ")
        p = all[0]
        arch = all[1]

        @installer.try_base_package_delete p, arch
    end
  end

  def handle_included_classes
    find_dependencies = lambda do |input_classes|
      out = []
      input_classes.each_with_index do |klass, index|
        if File.exist? "#{@fai_config_space}/class/#{klass}.deps"
          dependencies = File.readlines "#{@fai_config_space}/class/#{klass}.deps"
          dependencies.map! &:strip
          out.insert index, dependencies
          deps_of_deps = find_dependencies.call dependencies
          out.insert index, deps_of_deps
        end
        out << klass
      end

      out
    end

    # Find out all dependency classes.
    dependencies = find_dependencies.call @included_classes
    dependencies = dependencies.flatten.uniq.sort

=begin
    @included_classes.each_with_index do |klass, index|
      if File.exist? "#{@fai_config_space}/class/#{klass}.deps"
        dependencies = File.readlines "#{@fai_config_space}/class/#{klass}.deps"
        deps_of_deps = find_dependencies.call dependencies
        out.insert index, dependencies
      end
    end
=end

    @classes = dependencies
  end

  def set_classes

    if @included_classes
      handle_included_classes
    else
      # All available file names are classes.
      classes = execvalue %{cd #{@fai_config_space}/package_config; ls -1 | grep -E -i "^[a-zA-Z0-9_-]+$"}
      classes = classes.split "\n"
      addclasses = execvalue %{grep -h PACKAGES #{@fai_config_space}/package_config/* | sed -e 's/#.*//' | awk '{printf $3"\\n"$4"\\n"$5"\\n"$6"\\n"}'}, fail: true
      addclasses = addclasses.split "\n"
      #@classes = execvalue %{echo -e "#{classes}\n#{addclasses}\n" | sort | uniq}
      @classes = (classes + addclasses).sort.uniq
      @classes.delete_if do |c|
        c.empty?
      end
    end
    ENV["classes"] = @classes.join "\n"

    exclude_classes @excluded_classes if !@excluded_classes.nil? and !@excluded_classes.empty?

    verb "Using following classes in mirror:"
    verb @classes.join " "
    verb "Excluded classes: #{@excluded_classes}"
  end


  # create a list of files named class/*.var that contain a definition of a  variable that is
  # used in package_config/<classes>

  # we assume, that variables used in package_config are defined in class/*.var.
  # For all classes search if there's a variables used in package_config/CLASS
  # For all variables found look which files class/CLASS.var define these variables.
  def mk_class_var_list
    filelist = ""

    #filelist: all files in package_config which are also a defined class
    @classes.each do |c|
      if File.exist? "#{@fai_config_space}/package_config/#{c}"
        filelist += " #{c}"
      end
    end

    return if filelist.empty?

    # make the list of variables that are used
    # regex: $abc or ${abc}
    variables = execvalue(%{($(cd #{@fai_config_space}/package_config; grep -h -P -o '\$[A-Za-z0-9_]+|\$\{[A-Za-z0-9_]+\}' #{filelist} | sort | uniq))}).split("\n")

    # debug
    #variables.each do |variable|
    #  debug "Variable found: #{variable}"
    #end

    # create a regex like varname= for all variables

    variables.map! do |variable|
      variable.gsub! /[{}$]/, ""
      variable += "="
    end

    classlist = []

    # classlist: make list of files in class, which are also a class
    @classes.each do |c|
      if File.exist? "#{@fai_config_space}/class/#{c}.var"
        classlist << "#{@fai_config_space}/class/#{c}.var"
      end
    end
    # look for variable definition in this list

    sourcefiles = []

    classlist.each do |f|
      variables.each do |variable|
        execute %[grep -q -P "#{variable}" #{f}], log: false, fail: false
        if exec_success?
          sourcefiles << f
          break
        end
      end
    end

    # these files contain those variable definitions which are needed
    sourcefiles
  end
end