require_relative "mirror-impl"
require_relative "../packaging/packages"
require_relative "../ft"

class CreateFaiMirror::MirrorSuse < CreateFaiMirror::MirrorImpl

  def _post_initialize mirror_directory
    raise "For some reason mirror_directory is invalid..." if mirror_directory.empty?

    reposd_dir = "#{@fai_config_dir}/zypper/repos.d"
    unless Dir.exist? reposd_dir
      raise "FAI config dir does not contain #{@fai_config_dir}/zypper/repos.d/ for repositories used for the mirror."
    end

    raise "*SUSE distributions requires --target-distribution-version." unless @target_distribution_version

    #--cache-dir #{mirror_directory}
    #--reposd-dir #{reposd_dir}

    # TODO: the arguments really should be wrapped so that there can’t be funny arguments. Maybe execute() needs to support arrays for arguments.
    # NOTE: --gpg-auto-import-keys is used for convenience, as I can’t see other but hacky scripting to do proper key inserts.
    @zypper_options = <<~EOF
    --gpg-auto-import-keys
    --releasever "#{@target_distribution_version}"
    --non-interactive
    --non-interactive-include-reboot-patches
    --root "#{mirror_directory}"
    --reposd-dir "#{reposd_dir}"
    EOF
    @zypper_options.gsub!("\n", " ").strip!

    @mirror_directory = mirror_directory
  end

  # Installs packages to the mirror.
  def download_packages package_list
    env = { "LC_ALL" => "C" }
    result = execute %[#{pm_command} download -- #{package_list.join(" ")}], fail: false, env: env
    if result.match? /^Warning: No repositories defined./
      error "Internal error: for some reason zypper repository data is not defined properly."
    end
    if result.match? /^Warning: Argument resolves to no package:/
      error "Were not able to download all required packages. Please see above for list of packages."
    end
    if $?.exitstatus == 106
      puts "WARNING: a repository sync failed. It may mean that this command needs to be run as root."
    end
  end

  def refresh_repositories
    execute "#{pm_command} update"
  end

  def startup_checks
    # Nya.
  end

  # Add packages from base.tar.xz and additional packages in nfsroot.
  def add_base_packages
    # arch dependent packages defined in fai-make-nfsroot
    verb "Adding packages of #{@fai_config_dir}/NFSROOT."
    if File.exist? "#{nfsroot}/var/tmp/packages.nfsroot"
        plist = File.readlines("#{nfsroot}/var/tmp/packages.nfsroot")[0].split(" ")
        download_packages plist
    else
        STDERR.puts "WARNING: #{nfsroot}/var/tmp/packages.nfsroot does not exist."
        STDERR.puts "Can't add those packages. Maybe the nfsroot is not yet created."
        exit 1
    end

    use_temporary_sources_list

    base_packages_file = "#{nfsroot}/var/tmp/base-pkgs.lis"
    verb "Adding packages from #{base_packages_file}"
    if File.exist? base_packages_file
        plist = File.readlines base_packages_file
        plist.map! do |package|
          package.strip
        end
        refresh_repositories
        download_packages plist
    else
        STDERR.puts "WARNING: #{base_packages_file} does not exist."
        STDERR.puts "Can't add those packages. There is some problem with your nfsroot."
        exit 1
    end

    restore_sources_list
  end

  def download_packages_for_classes classes, fai_config_space
    verb "Downloading packages for classes: #{classes}"

    # This script is replacement of install_packages I made to work better with SUSE.
    ENV["FAI"] ||= fai_config_space
    ENV["FAI_ROOT"] = (ENV["FAI_ROOT"].nil? or ENV["FAI_ROOT"] == "/") && "/" || @mirror_directory
    ENV["PM_COMMAND_OPTIONS"] = @zypper_options
    #ENV["INSTALL_COMMAND_OPTIONS"] = @zypper_options
    # classes is defined by CreateFaiMirror::Methods#exclude_classes

    ARGV << @global_options if @global_options
    ARGV << "packages"
    ARGV << "--download-only"
    ARGV << "--target-distribution" << opts[:target_distribution]
    ARGV.flatten!

    debug ""
    debugstr = @@_debug && "(at #{caller_locations(0,1)[0]})" || ""
    debug %{Running “env classes=#{ENV["classes"].gsub("\n", "\\n")} FAI="#{ENV["FAI"]}" FAI_ROOT="#{ENV["FAI_ROOT"]}" PM_COMMAND_OPTIONS='#{ENV["PM_COMMAND_OPTIONS"]}' ft #{ARGV.join(" ")}” #{debugstr}}
    debug ""

    ft = FT.new
    ft.options
    ft.perform
  end

  # Hooks to make it visually easier to understand the flow.

  def _pre_real_perform fai_config_dir, mirror_location
    # These probably are not needed.
  end

  def _post_real_perform mirror_directory
    # These probably are not needed.
  end

  # Quasi-hooks

  def use_temporary_sources_list
    # These probably are not needed.
  end

  def restore_sources_list
    # These probably are not needed.
  end

  def try_base_package_delete p, arch
    # These probably are not needed.
  end

  private

  def pm_command
    %[env LC_ALL="C" zypper #{@zypper_options}]
  end
end