require "fileutils"

require_relative "nfsroot"
require_relative "scripts"
require_relative "../common"

class Nfsroot::NfsrootImpl
  include Common
  include Nfsroot::Scripts

  attr_accessor :fai_config_dir, :all_packages, :nonfree, :base_archive
  attr_accessor :ft_output_flags, :allow_run_scripts
  attr_accessor :base_compress_algo

  def initialize opts, config
    @opts = opts
    @config = config
  end

  private

  def extract_base base_archive
    unless File.exist? base_archive
      error "Base archive #{base_archive} could not be used."
    end

    Dir.chdir nfsroot do
      verb "Extracting base archive #{base_archive}: "

      case File.basename(base_archive).split(".", 2)[1]
      when "tar.gz", ".tgz"
        execute %{tar --xattrs --acls --xattrs-include=* -C "#{@config.nfsroot}" -zxpf "#{base_archive}"}
        FileUtils.cp base_archive, "#{@config.nfsroot}/var/tmp/base.tar.gz", preserve: true
      when "tar.xz", "txz"
        execute %{tar --xattrs --acls --xattrs-include=* -C "#{@config.nfsroot}" -Jxpf "#{base_archive}"}
        FileUtils.cp base_archive, "#{@config.nfsroot}/var/tmp/base.tar.xz", preserve: true
      when "tar.lz4"
        execute %{unlz4 "#{base_archive}" | tar --xattrs --acls --xattrs-include=* -C "#{@config.nfsroot}" -xp}
        FileUtils.cp base_archive, "#{@config.nfsroot}/var/tmp/base.tar.xz", preserve: true
      else
        error "Unrecognized base archive format in: #{base_archive}"
      end
    end
  end
end