require_relative "nfsroot"
require_relative "../common"
require_relative "../base/nfsroot-base-methods"

module Nfsroot::Scripts
  include Common

  private

  # Runs scripts inside FAI config space’s nfsroot/scripts dir.
  #
  # WARNING: This does not utilize FAI’s fai-do-scripts nor fully mimics how it works.
  def run_scripts hook: nil
    unless @allow_run_scripts
      debug "Skipping nfsroot scripts"
      return
    end

    env = {
      # Not using rootcmd() to avoid quotation problem. It can be used when execute() supports arrays for commands.
      "ROOTCMD" => %{chroot #{@@nfsroot} env LC_ALL=C},
      "NFSROOT" => @config.nfsroot,
      "FAI_CONFIG_DIR" => @fai_config_dir
    }

    parse_script = Proc.new do |i, env|
      # With hook == nil, there can be hook directories, that should be skipped.
      next if File.directory? i

      name = File.basename i
      suffix = i[-3,3]
      # NOTE: No special features has been done for shell scripts. Maybe
      # they could all be run in one sesion, for example. Or the env vars could be collected?
      if suffix == ".sh"
        verb "Executing shell script #{name}"
        execute i, env: env
        next
      end

      if suffix == ".rb"
        verb "Loading Ruby script #{name}"
        load i
        next
      end

      verb "Executing custom script #{name}"
      execute i, env: env
    end

    verb "#{hook || "scripts"}: Executing common scripts..."
    scripts_path = "#{@config.config_space}/nfsroot/scripts"
    scripts_path = "#{scripts_path}/#{hook}" if hook
    for i in Dir.glob("#{scripts_path}/*")
      parse_script.call i, env
    end

    verb "#{hook || "scripts"}: Executing fai config dir scripts..."
    scripts_path = "#{@fai_config_dir}/scripts"
    scripts_path = "#{scripts_path}/#{hook}" if hook
    for i in Dir.glob("#{scripts_path}/*")
      parse_script.call i, env
    end
  end
end