
require_relative "common"

# Generic option handling related methods.
module Options
  include Common

  def parse_global_options global_options
    @global_options = global_options
    @logging = global_options[:logging]
    set_opts global_options
    parse_common_options global_options
  end

  def options= input_options
    @options = input_options

    # Hacky thing so that we don’t accidentally unverbose a global verbose flag.
    @@_verbose = true if @options[:verbose]
    @@verbose = "-v" if @options[:verbose]
    @@_debug = true if @options[:debug]
    @@_trace = true if @options[:trace]
  end

  private

  # Outputs ft.rb compliant output flags, if they’ve been given as cli argument.
  #
  # @returns array of output flags
  def ft_output_flags
    out = []
    out << "--verbose" if @@_verbose
    out << "--debug" if @@_debug
    out
  end

  def common_options parser: nil, log_file: nil, &block
    if parser.nil?
      parser = Optimist::Parser.new
      created_parser = true
    end

    yield parser

    unless log_file.nil?
      parser.opt :log_dir,         "Directory to save logfiles to",
          type: :string,    short: :none, default: "/var/log/fai"
      parser.opt :log_file,        "Filename of the log",
          type: :string,    short: :none, default: log_file
    end

    parser.opt :trace,             "Show all of debug output",
                            short: :none
    parser.opt :debug,             "Show debug output",
                            short: "d"
    parser.opt :verbose,           "Show verbose output",
                            short: "v"

    if created_parser

      options = Optimist::with_standard_exception_handling parser do
        args = ARGV.clone

        result = parser.parse ARGV

        if not @run_by_default
          raise Optimist::HelpNeeded if args.empty?
        end

        result
      end

      return options
    end

    nil
  end

  def parse_common_options options
    unless options[:log_file].nil?
      log_dir = options[:log_dir]
      log_file = options[:log_file]

      # NOTE: If there is no permissions, this probably throws an error.
      Dir.mkdir options[:log_dir] unless Dir.exist? options[:log_dir]

      log_to_file "#{log_dir}/#{log_file}"
    end

    if options.has_key? :fai_config_dir and not options[:fai_config_dir].nil?
      raise "Given --fai-config-dir is not valid." if not Dir.exist? options[:fai_config_dir]
      @fai_config_dir = File.expand_path(options[:fai_config_dir])

      ensure_clean_state @fai_config_dir
    end
  end
end
