
# Outputting utilities
module Output

  @@_verbose = false
  @@_debug = false
  @@_trace = false

  # Everything should only be usable for the class inheriting this module.
  private

  def trace str
    return unless @@_trace
    out = "#{color(230, 165, 240)}[TRACE]#{stopcolor} #{str}"
    out = "#{out} (called at #{caller_locations(1,1)[0]})" if @@_trace
    puts out
    @log_file&.puts "[TRACE] #{str}"
  end

  def debug str
    return unless @@_debug
    out = "#{color(190, 190, 190)}[DEBUG]#{stopcolor} #{str}"
    out = "#{out} (called at #{caller_locations(1,1)[0]})" if @@_trace
    puts out
    @log_file&.puts "[DEBUG] #{str}"
  end

  def verb str
    return unless @@_verbose
    out = "#{color(130, 230, 130, bold: true)}[INFO]#{stopcolor} #{str}"
    out = "#{out} (called at #{caller_locations(1,1)[0]})" if @@_trace
    puts out
    @log_file&.puts str
  end

  def warn str
    out = "#{color(190, 190, 110, bold: true)}[WARN]#{stopcolor} #{str}"
    out = "#{out} (called at #{caller_locations(1,1)[0]})" if @@_debug or @@_trace
    STDERR.puts out
    @log_file&.puts str
  end

  def error str, exitcode: 1
    out = "#{color(230, 110, 110, bold: true)}[ERROR]#{stopcolor} #{str}"
    out = "#{out} (called at #{caller_locations(1,1)[0]})" if @@_debug or @@_trace
    STDERR.puts out
    @log_file&.puts str
    exit exitcode
  end

  def color red, green, blue, bold: false
    return if ENV["COLORTERM"] != "truecolor" && ENV["COLORTERM"] != "24bit"

    out = ""
    out += "\x1b[38;1;1m" if bold
    # https://stackoverflow.com/questions/4842424/list-of-ansi-color-escape-sequences
    out += "\x1b[38;2;#{red};#{green};#{blue}m"

    out
  end

  def stopcolor
    return if ENV["COLORTERM"] != "truecolor" && ENV["COLORTERM"] != "24bit"

    "\x1b[0m"
  end

  def log_to_file log_path
    return unless @logging

    log_file = File.open log_path, "w"

    at_exit do
      log_file.close
    end

    @log_file = log_file
  end
end
